Method: HexaPDF::Content::Canvas#end_path

Defined in:
lib/hexapdf/content/canvas.rb

#end_pathObject

:call-seq:

canvas.end_path     => canvas

Ends the path without stroking or filling it and returns self.

This method is usually used in conjunction with the clipping path methods to define the clipping path.

Examples:

canvas.line(10, 10, 100, 100)
canvas.end_path                    # Nothing to see here!

See: PDF2.0 s8.5.3.1, #clip_path



1669
1670
1671
1672
1673
# File 'lib/hexapdf/content/canvas.rb', line 1669

def end_path
  raise_unless_in_path_or_clipping_path
  invoke0(:n)
  self
end