Method: HexaPDF::Content::Processor::GlyphBox#upper_right

Defined in:
lib/hexapdf/content/processor.rb

#upper_rightObject

:call-seq:

glyph_box.upper_right    -> [urx, ury]

Returns the upper right coordinate which is computed by using the other three points of the parallelogram.



136
137
138
# File 'lib/hexapdf/content/processor.rb', line 136

def upper_right
  [@ulx + (@lrx - @llx), @uly + (@lry - @lly)]
end