Method: HexaPDF::Type::AcroForm::TextField#field_value

Defined in:
lib/hexapdf/type/acro_form/text_field.rb

#field_valueObject

Returns the field value, i.e. the text contents of the field, or nil if no value is set.

Note that modifying the returned value *might not* modify the text contents in case it is stored as stream! So always use #field_value= to set the field value.



160
161
162
163
# File 'lib/hexapdf/type/acro_form/text_field.rb', line 160

def field_value
  return unless value[:V]
  self[:V].kind_of?(String) ? self[:V] : self[:V].stream
end