Class: HexaPDF::CLI::Command
- Inherits:
-
CmdParse::Command
- Object
- CmdParse::Command
- HexaPDF::CLI::Command
- Includes:
- Extensions
- Defined in:
- lib/hexapdf/cli/command.rb
Overview
Base class for all hexapdf commands. It provides utility methods needed by the individual commands.
Direct Known Subclasses
Batch, DebugInfo, Files, Fonts, Form, Image2PDF, Images, Info, Inspect, Merge, Modify, Optimize, Split, Usage, Watermark
Defined Under Namespace
Modules: Extensions
Instance Method Summary collapse
-
#initialize(*args, **kwargs, &block) ⇒ Command
constructor
A new instance of Command.
Methods included from Extensions
Constructor Details
#initialize(*args, **kwargs, &block) ⇒ Command
Returns a new instance of Command.
68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 |
# File 'lib/hexapdf/cli/command.rb', line 68 def initialize(*args, **kwargs, &block) #:nodoc: super @out_options = { compact: true, compress_pages: false, object_streams: :preserve, xref_streams: :preserve, streams: :preserve, optimize_fonts: false, prune_page_resources: false, encryption: :preserve, enc_user_pwd: nil, enc_owner_pwd: nil, enc_key_length: 128, enc_algorithm: :aes, enc_force_v4: false, enc_permissions: [], } end |