Class: HexaPDF::Content::ColorSpace::DeviceCMYK

Inherits:
Object
  • Object
show all
Defined in:
lib/hexapdf/content/color_space.rb

Overview

The DeviceCMYK color space.

Defined Under Namespace

Classes: Color

Constant Summary collapse

DEFAULT =

The one (and only) DeviceCMYK color space.

new

Class Method Summary collapse

Instance Method Summary collapse

Class Method Details

.new(_definition = nil) ⇒ Object

Returns the DeviceCMYK color space object.



508
509
510
# File 'lib/hexapdf/content/color_space.rb', line 508

def self.new(_definition = nil)
  DEFAULT
end

Instance Method Details

#color(c, m, y, k) ⇒ Object

Returns the color object for the given cyan, magenta, yellow and black components.

Color values can either be integers in the range from 0 to 100 or floating point numbers between 0.0 and 1.0. The integer color values are automatically normalized to the DeviceCMYK color value range of 0.0 to 1.0.



522
523
524
525
# File 'lib/hexapdf/content/color_space.rb', line 522

def color(c, m, y, k)
  Color.new(ColorUtils.normalize_value(c, 100), ColorUtils.normalize_value(m, 100),
            ColorUtils.normalize_value(y, 100), ColorUtils.normalize_value(k, 100))
end

#default_colorObject

Returns the default color for the DeviceCMYK color space.



513
514
515
# File 'lib/hexapdf/content/color_space.rb', line 513

def default_color
  Color.new(0.0, 0.0, 0.0, 1.0)
end

#familyObject Also known as: definition

Returns :DeviceCMYK.



536
537
538
# File 'lib/hexapdf/content/color_space.rb', line 536

def family
  :DeviceCMYK
end

#prenormalized_color(c, m, y, k) ⇒ Object

Returns the color object for the cyan, magenta, yellow and black components without applying value normalization.

See: #color



531
532
533
# File 'lib/hexapdf/content/color_space.rb', line 531

def prenormalized_color(c, m, y, k)
  Color.new(c, m, y, k)
end