Class: HexaPDF::Encryption::EncryptionDictionary

Inherits:
Dictionary
  • Object
show all
Defined in:
lib/hexapdf/encryption/security_handler.rb

Overview

Base class for all encryption dictionaries.

Contains entries common to all encryption dictionaries. If a specific security handler needs further fields it should derive a new subclass and add the new fields there.

See: PDF2.0 s7.6.2

Direct Known Subclasses

StandardEncryptionDictionary

Constant Summary

Constants included from DictionaryFields

DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate

Instance Method Summary collapse

Methods inherited from Dictionary

#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type

Instance Method Details

#must_be_indirect?Boolean

Returns true because some PDF readers stumble when encountering a non-indirect encryption dictionary.

Returns:



64
65
66
# File 'lib/hexapdf/encryption/security_handler.rb', line 64

def must_be_indirect?
  true
end