Class: HexaPDF::Encryption::StandardEncryptionDictionary
- Inherits:
-
EncryptionDictionary
- Object
- Object
- Dictionary
- EncryptionDictionary
- HexaPDF::Encryption::StandardEncryptionDictionary
- Defined in:
- lib/hexapdf/encryption/standard_security_handler.rb
Overview
The specialized encryption dictionary for the StandardSecurityHandler.
Contains additional fields that are used for storing the information needed for retrieving the encryption key and a set of permissions.
Constant Summary
Constants included from DictionaryFields
DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate
Method Summary
Methods inherited from EncryptionDictionary
Methods inherited from Dictionary
#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type