Method: HexaPDF::Layout::ImageBox#initialize
- Defined in:
- lib/hexapdf/layout/image_box.rb
#initialize(image:, **kwargs) ⇒ ImageBox
Creates a new Image box object for the given image argument which needs to be an image object (e.g. returned by HexaPDF::Document::Images#add).
72 73 74 75 |
# File 'lib/hexapdf/layout/image_box.rb', line 72 def initialize(image:, **kwargs) super(**kwargs) @image = image end |