Method: HexaPDF::Layout::Line#x_offset

Defined in:
lib/hexapdf/layout/line.rb

#x_offsetObject

An optional horizontal offset that should be taken into account when positioning the line.

This offset always describes the offset from the left side (and not, for example, the offset from the right side of another line even if those two lines are actually on the same horizontal level).



180
181
182
# File 'lib/hexapdf/layout/line.rb', line 180

def x_offset
  @x_offset
end