Class: HexaPDF::Object

Inherits:
Object
  • Object
show all
Includes:
Comparable
Defined in:
lib/hexapdf/object.rb

Overview

Objects of the PDF object system.

Overview

A PDF object is like a normal object but with an additional *object identifier* consisting of an object number and a generation number. If the object number is zero, then the PDF object represents a direct object. Otherwise the object identifier uniquely identifies this object as an indirect object and can be used for referencing it (from possibly multiple places).

Furthermore a PDF object may have an associated stream. However, this stream is only accessible if the subclass Stream is used.

A PDF object should be connected to a PDF document, otherwise some methods may not work.

Most PDF objects in a PDF document are represented by subclasses of this class that provide additional functionality.

The methods #hash and #eql? are implemented so that objects of this class can be used as hash keys. Furthermore the implementation is compatible to the one of Reference, i.e. the hash of a PDF Object is the same as the hash of its corresponding Reference object.

Allowed PDF Object Values

The PDF specification knows of the following object types:

  • Boolean (mapped to true and false),

  • Integer (mapped to Integer object)

  • Real (mapped to Float objects)

  • String (mapped to String objects with UTF-8 or binary encoding)

  • Names (mapped to Symbol objects)

  • Array (mapped to Array objects)

  • Dictionary (mapped to Hash objects)

  • Stream (mapped to the Stream class which is a Dictionary with the associated stream data)

  • Null (mapped to nil)

  • Indirect Object (mapped to this class)

So working with PDF objects in HexaPDF is rather straightforward since the common Ruby objects can be used for most things, i.e. wrapping an plain Ruby object into an object of this class is not necessary (except if it should become an indirect object).

There are also some additional data structures built from these primitive ones. For example, Time objects are represented as specially formatted string objects and conversion from and to the string representation is handled automatically.

Important: Users of HexaPDF may use other plain Ruby objects but then there is no guarantee that everything will work correctly, especially when using other collection types than arrays and hashes.

See: HexaPDF::Dictionary, HexaPDF::Stream, HexaPDF::Reference, HexaPDF::Document

See: PDF1.7 s7.3.10, s7.3.8

Direct Known Subclasses

Dictionary, PDFArray

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(value, document: nil, oid: nil, gen: nil, stream: nil) ⇒ Object

Creates a new PDF object wrapping the value.

The value can either be a PDFData object in which case it is used directly. If it is a PDF Object, then its data is used. Otherwise the value object is used as is. In all cases, the oid, gen and stream values may be overridden by the corresponding keyword arguments.



178
179
180
181
182
183
184
185
186
187
188
189
190
# File 'lib/hexapdf/object.rb', line 178

def initialize(value, document: nil, oid: nil, gen: nil, stream: nil)
  @data = case value
          when PDFData then value
          when Object then value.data
          else PDFData.new(value)
          end
  @data.oid = oid if oid
  @data.gen = gen if gen
  @data.stream = stream if stream
  self.document = document
  self.must_be_indirect = false
  after_data_change
end

Instance Attribute Details

#dataObject (readonly)

The wrapped HexaPDF::PDFData value.

This attribute is not part of the public API!



165
166
167
# File 'lib/hexapdf/object.rb', line 165

def data
  @data
end

#documentObject

Returns the associated PDF document.

If no document is associated, an error is raised.



226
227
228
# File 'lib/hexapdf/object.rb', line 226

def document
  @document || raise(HexaPDF::Error, "No document associated with this object (#{inspect})")
end

#must_be_indirect=(value) ⇒ Object (writeonly)

Sets whether the object has to be an indirect object once it is written.



171
172
173
# File 'lib/hexapdf/object.rb', line 171

def must_be_indirect=(value)
  @must_be_indirect = value
end

Class Method Details

.deep_copy(object) ⇒ Object

:call-seq:

HexaPDF::Object.deep_copy(object)    -> copy

Creates a deep copy of the given object which retains the references to indirect objects.



129
130
131
132
133
134
135
136
137
138
139
140
141
142
# File 'lib/hexapdf/object.rb', line 129

def self.deep_copy(object)
  case object
  when Hash
    object.transform_values {|value| deep_copy(value) }
  when Array
    object.map {|o| deep_copy(o) }
  when HexaPDF::Object
    (object.indirect? || object.must_be_indirect? ? object : deep_copy(object.value))
  when HexaPDF::Reference
    object
  else
    object.dup
  end
end

.make_direct(object) ⇒ Object

Makes sure that the object itself as well as all nested values are direct objects.

If an indirect object is found, it is turned into a direct object and the indirect object is deleted from the document.



148
149
150
151
152
153
154
155
156
157
158
159
160
# File 'lib/hexapdf/object.rb', line 148

def self.make_direct(object)
  if object.kind_of?(HexaPDF::Object) && object.indirect?
    object_to_delete = object
    object = object.value
    object_to_delete.document.delete(object_to_delete)
  end
  if object.kind_of?(Hash)
    object.transform_values! {|val| make_direct(val) }
  elsif object.kind_of?(Array)
    object.map! {|val| make_direct(val) }
  end
  object
end

Instance Method Details

#<=>(other) ⇒ Object

Compares this object to another object.

If the other object does not respond to oid or gen, nil is returned. Otherwise objects are ordered first by object number and then by generation number.



331
332
333
334
# File 'lib/hexapdf/object.rb', line 331

def <=>(other)
  return nil unless other.respond_to?(:oid) && other.respond_to?(:gen)
  (oid == other.oid ? gen <=> other.gen : oid <=> other.oid)
end

#==(other) ⇒ Object

Returns true if the other object is an Object and wraps the same #data structure, if the other object is a Reference with the same oid/gen, or if this object is not indirect and its value is equal to the other object.



339
340
341
342
# File 'lib/hexapdf/object.rb', line 339

def ==(other)
  (other.kind_of?(Object) && data == other.data) || (other.kind_of?(Reference) && other == self) ||
    (!indirect? && other == data.value)
end

#cache(key, value = Document::UNSET, update: false, &block) ⇒ Object

Caches and returns the given value or the value of the block under the given cache key. If there is already a cached value for the key and update is false, it is just returned.

Set update to true to force an update of the cached value.

This uses Document#cache internally.



311
312
313
# File 'lib/hexapdf/object.rb', line 311

def cache(key, value = Document::UNSET, update: false, &block)
  document.cache(@data, key, value, update: update, &block)
end

#cached?(key) ⇒ Boolean

Returns true if there is a cached value for the given key.

This uses Document#cached? internally.

Returns:

  • (Boolean)


318
319
320
# File 'lib/hexapdf/object.rb', line 318

def cached?(key)
  document.cached?(@data, key)
end

#clear_cacheObject

Clears the cache for this object.



323
324
325
# File 'lib/hexapdf/object.rb', line 323

def clear_cache
  document.clear_cache(@data)
end

#deep_copyObject

Makes a deep copy of the source PDF object and resets the object identifier.



295
296
297
298
299
300
301
302
303
# File 'lib/hexapdf/object.rb', line 295

def deep_copy
  obj = dup
  obj.instance_variable_set(:@data, @data.dup)
  obj.data.oid = 0
  obj.data.gen = 0
  obj.data.stream = @data.stream.dup if @data.stream.kind_of?(String)
  obj.data.value = self.class.deep_copy(@data.value)
  obj
end

#document?Boolean

Returns true if a PDF document is associated.

Returns:

  • (Boolean)


231
232
233
# File 'lib/hexapdf/object.rb', line 231

def document?
  !@document.nil?
end

#eql?(other) ⇒ Boolean

Returns true if the other object references the same PDF object as this object.

Returns:

  • (Boolean)


345
346
347
# File 'lib/hexapdf/object.rb', line 345

def eql?(other)
  other.respond_to?(:oid) && oid == other.oid && other.respond_to?(:gen) && gen == other.gen
end

#genObject

Returns the generation number of the PDF object.



203
204
205
# File 'lib/hexapdf/object.rb', line 203

def gen
  data.gen
end

#gen=(gen) ⇒ Object

Sets the generation number of the PDF object.



208
209
210
# File 'lib/hexapdf/object.rb', line 208

def gen=(gen)
  data.gen = gen
end

#hashObject

Computes the hash value based on the object and generation numbers.



350
351
352
# File 'lib/hexapdf/object.rb', line 350

def hash
  oid.hash ^ gen.hash
end

#indirect?Boolean

Returns true if the object is an indirect object (i.e. has an object number unequal to zero).

Returns:

  • (Boolean)


237
238
239
# File 'lib/hexapdf/object.rb', line 237

def indirect?
  oid != 0
end

#inspectObject

:nodoc:



354
355
356
# File 'lib/hexapdf/object.rb', line 354

def inspect #:nodoc:
  "#<#{self.class.name} [#{oid}, #{gen}] value=#{value.inspect}>"
end

#must_be_indirect?Boolean

Returns true if the object must be an indirect object once it is written.

Returns:

  • (Boolean)


242
243
244
# File 'lib/hexapdf/object.rb', line 242

def must_be_indirect?
  @must_be_indirect
end

#null?Boolean

Returns true if the object represents the PDF null object.

Returns:

  • (Boolean)


262
263
264
# File 'lib/hexapdf/object.rb', line 262

def null?
  value.nil?
end

#oidObject

Returns the object number of the PDF object.



193
194
195
# File 'lib/hexapdf/object.rb', line 193

def oid
  data.oid
end

#oid=(oid) ⇒ Object

Sets the object number of the PDF object.



198
199
200
# File 'lib/hexapdf/object.rb', line 198

def oid=(oid)
  data.oid = oid
end

#typeObject

Returns the type (symbol) of the object.

Since the type system is implemented in such a way as to allow exchanging implementations of specific types, the class of an object can't be reliably used for determining the actual type.

However, the Type and Subtype fields can easily be used for this. Subclasses for PDF objects that don't have such fields may use a unique name that has to begin with XX (see PDF1.7 sE.2) and therefore doesn't clash with names defined by the PDF specification.

For basic objects this always returns :Unknown.



257
258
259
# File 'lib/hexapdf/object.rb', line 257

def type
  :Unknown
end

#validate(auto_correct: true) ⇒ Object

:call-seq:

obj.validate(auto_correct: true)                                    -> true or false
obj.validate(auto_correct: true) {|msg, correctable, obj| block }   -> true or false

Validates the object, optionally corrects problems when the option auto_correct is set and returns true if the object is deemed valid and false otherwise.

If a block is given, it is called on validation problems with a problem description and whether the problem is automatically correctable. The third argument to the block is usually this object but may be another object if during auto-correction a new object was created and validated.

The validation routine itself has to be implemented in the #perform_validation method - see its documentation for more information.

Note: Even if the return value is true there may be problems since HexaPDF doesn't currently implement the full PDF spec. However, if the return value is false, there is certainly a problem!



284
285
286
287
288
289
290
291
292
# File 'lib/hexapdf/object.rb', line 284

def validate(auto_correct: true)
  result = true
  perform_validation do |msg, correctable, object|
    yield(msg, correctable, object || self) if block_given?
    result = false unless correctable
    return false unless auto_correct
  end
  result
end

#valueObject

Returns the object value.



213
214
215
# File 'lib/hexapdf/object.rb', line 213

def value
  data.value
end

#value=(val) ⇒ Object

Sets the object value. Unlike in #initialize the value is used as is!



218
219
220
221
# File 'lib/hexapdf/object.rb', line 218

def value=(val)
  data.value = val
  after_data_change
end