Class: HexaPDF::PDFData
- Inherits:
-
Object
- Object
- HexaPDF::PDFData
- Defined in:
- lib/hexapdf/object.rb
Overview
Internal value object for storing object number, generation number, object value and a possible stream together. Such objects are not used directly but wrapped by Object or one of its subclasses.
Instance Attribute Summary collapse
Instance Method Summary collapse
-
#initialize(value, oid = nil, gen = nil, stream = nil) ⇒ PDFData
constructor
A new instance of PDFData.
Constructor Details
#initialize(value, oid = nil, gen = nil, stream = nil) ⇒ PDFData
Returns a new instance of PDFData.
53 54 55 56 57 58 |
# File 'lib/hexapdf/object.rb', line 53 def initialize(value, oid = nil, gen = nil, stream = nil) #:nodoc: self.value = value self.oid = oid self.gen = gen self.stream = stream end |