Class: HexaPDF::Type::AcroForm::SignatureField::LockDictionary

Inherits:
Dictionary
  • Object
show all
Defined in:
lib/hexapdf/type/acro_form/signature_field.rb

Overview

A signature field lock dictionary specifies a set of form fields that should be locked once the associated signature field is signed.

See: PDF2.0 s12.7.5.5

Constant Summary

Constants included from DictionaryFields

DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate

Method Summary

Methods inherited from Dictionary

#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type