Class: HexaPDF::Type::Action

Inherits:
Dictionary show all
Defined in:
lib/hexapdf/type/action.rb

Overview

Represents a generic PDF action dictionary.

Action dictionaries are used, for example, by annotations or outline items to specify the action that should be performed. Each action class should be defined under the Actions module.

See: PDF2.0 s12.6

Constant Summary

Constants included from DictionaryFields

DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate

Method Summary

Methods inherited from Dictionary

#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type