Class: HexaPDF::Type::EmbeddedFile

Inherits:
Stream show all
Defined in:
lib/hexapdf/type/embedded_file.rb

Overview

Represents an embedded file stream.

An embedded file stream contains the data of, and optionally some meta data about, a file that is embedded into the PDF file. Each embedded file is either associated with a certain Type::FileSpecification dictionary or with the document as a whole through the /EmbeddedFiles entry in the document catalog's /Names dictionary.

See: PDF2.0 s7.11.4, FileSpecification

Defined Under Namespace

Classes: MacInfo, Parameters

Constant Summary

Constants included from DictionaryFields

DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate

Method Summary

Methods inherited from Stream

#must_be_indirect?, #raw_stream, #set_filter, #stream, #stream=, #stream_decoder, #stream_encoder, #stream_source

Methods inherited from Dictionary

#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type