Class: HexaPDF::Type::Metadata

Inherits:
Stream show all
Defined in:
lib/hexapdf/type/metadata.rb

Overview

Represents an XMP metadata stream.

XMP metadata streams may be attached to most PDF objects, though it only makes sense for some of them.

There is also a main XMP metadata stream for the whole document that is accessible via the /Metadata key of the document catalog. That metadata stream should contain the same values as the PDF's info dictionary and may contain additional entries. This can be accomplished via HexaPDF::Document#metadata.

See: PDF2.0 s14.3.2

Constant Summary

Constants included from DictionaryFields

DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate

Method Summary

Methods inherited from Stream

#must_be_indirect?, #raw_stream, #set_filter, #stream, #stream=, #stream_decoder, #stream_encoder, #stream_source

Methods inherited from Dictionary

#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type