Class: HexaPDF::Type::Names

Inherits:
Dictionary show all
Defined in:
lib/hexapdf/type/names.rb

Overview

Represents the PDF's names dictionary which associates names with data for various purposes.

Each field corresponds to a name tree that holds the information and can be used to find, add or delete an entry.

This dictionary is linked via the /Names entry from the HexaPDF::Catalog.

See: PDF2.0 s7.7.4, HexaPDF::Catalog, HexaPDF::NameTreeNode

Constant Summary

Constants included from DictionaryFields

DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate

Instance Method Summary collapse

Methods inherited from Dictionary

#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type

Instance Method Details

#destinationsObject

Returns the destinations name tree containing a mapping from names to destination objects.

The name tree will be created if needed.

Note: It is possible to use this name tree directly, but HexaPDF::Document::Destinations provides a much easier to work with convenience interface for working with destination objects.

See: PDF2.0 s12.3.2



75
76
77
# File 'lib/hexapdf/type/names.rb', line 75

def destinations
  self[:Dests] ||= document.add({}, type: NameTreeNode)
end