Class: HexaPDF::Type::OptionalContentMembership
- Inherits:
-
Dictionary
- Object
- Object
- Dictionary
- HexaPDF::Type::OptionalContentMembership
- Defined in:
- lib/hexapdf/type/optional_content_membership.rb
Overview
Represents an optional content membership dictionary.
A membership dictionary allows more complex visibility policies, like:
- Content that should be visible when a certain optional content group is off instead of on.
- Content that should be visible when all of a number of OCGs are on.
See: PDF2.0 s8.11.2.2
Constant Summary
Constants included from DictionaryFields
DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate
Method Summary
Methods inherited from Dictionary
#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type