Class: HexaPDF::Type::StructTreeRoot
- Inherits:
-
Dictionary
- Object
- Object
- Dictionary
- HexaPDF::Type::StructTreeRoot
- Defined in:
- lib/hexapdf/type/struct_tree_root.rb
Overview
Represents the structure tree root dictionary, the root of the structure tree which is the value of /StructTreeRoot entry in the document catalog.
See: PDF2.0 s14.7.2
Constant Summary
Constants included from DictionaryFields
DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate
Method Summary
Methods inherited from Dictionary
#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type