Method: HexaPDF::Content::Canvas#current_point
- Defined in:
- lib/hexapdf/content/canvas.rb
#current_point ⇒ Object (readonly)
The current point [x, y] of the path.
This attribute holds the current point which is only valid if the current graphics object (see #graphic_object) is :path.
When the current point changes, the array is modified in place instead of creating a new array!
273 274 275 |
# File 'lib/hexapdf/content/canvas.rb', line 273 def current_point @current_point end |