Method: HexaPDF::Content::Canvas#end_text
- Defined in:
- lib/hexapdf/content/canvas.rb
#end_text ⇒ Object
:call-seq:
canvas.end_text -> canvas
Ends the current text object and returns self.
It is not necessary to invoke this method manually in most cases since it is automatically called when needed by other methods, i.e. when creating a new path.
See: PDF2.0 s9.4.1, #begin_text
2095 2096 2097 2098 2099 |
# File 'lib/hexapdf/content/canvas.rb', line 2095 def end_text raise_unless_at_page_description_level_or_in_text invoke0(:ET) if graphics_object == :text self end |