Method: HexaPDF::Content::Canvas#graphics_state
- Defined in:
- lib/hexapdf/content/canvas.rb
#graphics_state ⇒ Object (readonly)
The GraphicsState object containing the current graphics state.
The graphics state must not be changed directly, only by using the provided methods. If it is changed directly, the output will not be correct.
249 250 251 |
# File 'lib/hexapdf/content/canvas.rb', line 249 def graphics_state @graphics_state end |