Method: HexaPDF::Content::Canvas#pos
- Defined in:
- lib/hexapdf/content/canvas.rb
#pos(x, y) ⇒ Object
Returns the position (x,y) transformed by the current transformation matrix.
The resulting position should be interpreted in terms of the coordinate system of the context object (e.g. the page or Form XObject).
325 326 327 |
# File 'lib/hexapdf/content/canvas.rb', line 325 def pos(x, y) graphics_state.ctm.evaluate(x, y) end |