Method: HexaPDF::Content::Processor::GlyphBox#initialize
- Defined in:
- lib/hexapdf/content/processor.rb
#initialize(code_point, string, llx, lly, lrx, lry, ulx, uly) ⇒ GlyphBox
Creates a new glyph box for the given code point/Unicode value pair with the lower left coordinate [llx, lly], the lower right coordinate [lrx, lry], and the upper left coordinate [ulx, uly].
95 96 97 98 99 100 101 102 103 104 105 |
# File 'lib/hexapdf/content/processor.rb', line 95 def initialize(code_point, string, llx, lly, lrx, lry, ulx, uly) @code_point = code_point @string = string.freeze @llx = llx @lly = lly @lrx = lrx @lry = lry @ulx = ulx @uly = uly freeze end |