Method: HexaPDF::Encryption::StandardSecurityHandler::EncryptionOptions#owner_password
- Defined in:
- lib/hexapdf/encryption/standard_security_handler.rb
#owner_password ⇒ Object
The owner password. If this attribute is not specified but the virtual password attribute is, then the latter is used.
197 198 199 |
# File 'lib/hexapdf/encryption/standard_security_handler.rb', line 197 def owner_password @owner_password end |