Method: HexaPDF::Type::AcroForm::TextField#initialize_as_password_field
- Defined in:
- lib/hexapdf/type/acro_form/text_field.rb
#initialize_as_password_field ⇒ Object
Initializes the text field to be a password field.
This method should only be called directly after creating a new text field because it doesn’t completely reset the object.
121 122 123 124 125 |
# File 'lib/hexapdf/type/acro_form/text_field.rb', line 121 def initialize_as_password_field delete(:V) flag(:password) unflag(:comb, :multiline, :file_select) end |