Class: HexaPDF::Font::TrueType::Table::Hmtx
- Inherits:
-
Table
- Object
- Table
- HexaPDF::Font::TrueType::Table::Hmtx
- Defined in:
- lib/hexapdf/font/true_type/table/hmtx.rb
Overview
The 'hmtx' (horizontal metrics) table contains information for the horizontal layout of each glyph in the font.
See: https://developer.apple.com/fonts/TrueType-Reference-Manual/RM06/Chap6hmtx.html
Defined Under Namespace
Classes: Metric
Instance Attribute Summary collapse
-
#horizontal_metrics ⇒ Object
A hash of glyph ID to Metric objects mapping.
Instance Method Summary collapse
-
#[](glyph_id) ⇒ Object
Returns the Metric object for the give glyph ID.
Instance Attribute Details
#horizontal_metrics ⇒ Object
A hash of glyph ID to Metric objects mapping.
55 56 57 |
# File 'lib/hexapdf/font/true_type/table/hmtx.rb', line 55 def horizontal_metrics @horizontal_metrics end |
Instance Method Details
#[](glyph_id) ⇒ Object
Returns the Metric object for the give glyph ID.
58 59 60 |
# File 'lib/hexapdf/font/true_type/table/hmtx.rb', line 58 def [](glyph_id) @horizontal_metrics[glyph_id] end |