Class: HexaPDF::Layout::Box
- Inherits:
-
Object
- Object
- HexaPDF::Layout::Box
- Defined in:
- lib/hexapdf/layout/box.rb
Overview
The base class for all layout boxes.
Box Model
HexaPDF uses the following box model:
-
Each box can specify a width and height. Padding and border are inside, the margin outside of this rectangle.
-
The #content_width and #content_height accessors can be used to get the width and height of the content box without padding and the border.
-
If width or height is set to zero, they are determined automatically during layouting.
Subclasses
Each subclass should only take keyword arguments on initialization so that the boxes can be instantiated from the common convenience method HexaPDF::Document::Layout#box. To use this facility subclasses need to be registered with the configuration option 'layout.boxes.map'.
The methods #fit, #split or #split_content, and #draw or #draw_content need to be customized according to the subclass's use case.
- #fit
-
This method should return
trueif fitting was successful. Additionally, the @fit_successful instance variable needs to be set to the fit result as it is used in #split. - #split
-
This method splits the content so that the available space is used as good as possible. The default implementation should be fine for most use-cases, so only #split_content needs to be implemented. The method #create_split_box should be used for getting a basic cloned box.
- #draw
-
This method draws the content and the default implementation already handles things like drawing the border and background. Therefore it's best to implement #draw_content which should just draw the content.
Instance Attribute Summary collapse
-
#height ⇒ Object
readonly
The height of the box, including padding and/or borders.
-
#style ⇒ Object
readonly
The style to be applied.
-
#width ⇒ Object
readonly
The width of the box, including padding and/or borders.
Class Method Summary collapse
-
.create(width: 0, height: 0, content_box: false, style: nil, **style_properties, &block) ⇒ Object
Creates a new Box object, using the provided block as drawing block (see ::new).
Instance Method Summary collapse
-
#content_height ⇒ Object
The height of the content box, i.e.
-
#content_width ⇒ Object
The width of the content box, i.e.
-
#draw(canvas, x, y) ⇒ Object
Draws the content of the box onto the canvas at the position (x, y).
-
#empty? ⇒ Boolean
Returns
trueif no drawing operations are performed. -
#fit(available_width, available_height, _frame) ⇒ Object
Fits the box into the Frame and returns
trueif fitting was successful. -
#initialize(width: 0, height: 0, style: nil, &block) ⇒ Box
constructor
:call-seq: Box.new(width: 0, height: 0, style: nil) {|canv, box| block} -> box.
-
#split(available_width, available_height, frame) ⇒ Object
Tries to split the box into two, the first of which needs to fit into the available space, and returns the parts as array.
-
#split_box? ⇒ Boolean
Returns
trueif this is a split box, i.e. -
#supports_position_flow? ⇒ Boolean
Returns
falsesince a basic box doesn't support the 'position' style property value :flow.
Constructor Details
#initialize(width: 0, height: 0, style: nil, &block) ⇒ Box
:call-seq:
Box.new(width: 0, height: 0, style: nil) {|canv, box| block} -> box
Creates a new Box object with the given width and height that uses the provided block when it is asked to draw itself on a canvas (see #draw).
Since the final location of the box is not known beforehand, the drawing operations inside the block should draw inside the rectangle (0, 0, content_width, content_height) - note that the width and height of the box may not be known beforehand.
125 126 127 128 129 130 131 132 |
# File 'lib/hexapdf/layout/box.rb', line 125 def initialize(width: 0, height: 0, style: nil, &block) @width = @initial_width = width @height = @initial_height = height @style = Style.create(style) @draw_block = block @fit_successful = false @split_box = false end |
Instance Attribute Details
#height ⇒ Object (readonly)
The height of the box, including padding and/or borders.
102 103 104 |
# File 'lib/hexapdf/layout/box.rb', line 102 def height @height end |
#style ⇒ Object (readonly)
The style to be applied.
Only the following properties are used:
-
Style#background_color
-
Style#background_alpha
-
Style#padding
-
Style#border
-
Style#overlays
-
Style#underlays
114 115 116 |
# File 'lib/hexapdf/layout/box.rb', line 114 def style @style end |
#width ⇒ Object (readonly)
The width of the box, including padding and/or borders.
99 100 101 |
# File 'lib/hexapdf/layout/box.rb', line 99 def width @width end |
Class Method Details
.create(width: 0, height: 0, content_box: false, style: nil, **style_properties, &block) ⇒ Object
Creates a new Box object, using the provided block as drawing block (see ::new).
If content_box is true, the width and height are taken to mean the content width and height and the style's padding and border are added to them appropriately.
The style argument defines the Style object (see Style::create for details) for the box. Any additional keyword arguments have to be style properties and are applied to the style object.
87 88 89 90 91 92 93 94 95 96 |
# File 'lib/hexapdf/layout/box.rb', line 87 def self.create(width: 0, height: 0, content_box: false, style: nil, **style_properties, &block) style = Style.create(style).update(**style_properties) if content_box width += style.padding.left + style.padding.right + style.border.width.left + style.border.width.right height += style.padding.top + style.padding.bottom + style.border.width.top + style.border.width.bottom end new(width: width, height: height, style: style, &block) end |
Instance Method Details
#content_height ⇒ Object
The height of the content box, i.e. without padding and/or borders.
151 152 153 154 |
# File 'lib/hexapdf/layout/box.rb', line 151 def content_height height = @height - reserved_height height < 0 ? 0 : height end |
#content_width ⇒ Object
The width of the content box, i.e. without padding and/or borders.
145 146 147 148 |
# File 'lib/hexapdf/layout/box.rb', line 145 def content_width width = @width - reserved_width width < 0 ? 0 : width end |
#draw(canvas, x, y) ⇒ Object
Draws the content of the box onto the canvas at the position (x, y).
The coordinate system is translated so that the origin is at the bottom left corner of the **content box** during the drawing operations when @draw_block is used.
The block specified when creating the box is invoked with the canvas and the box as arguments. Subclasses can specify an on-demand drawing method by setting the @draw_block instance variable to nil or a valid block. This is useful to avoid unnecessary set-up operations when the block does nothing.
203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 |
# File 'lib/hexapdf/layout/box.rb', line 203 def draw(canvas, x, y) if style.background_color? && style.background_color canvas.save_graphics_state do canvas.opacity(fill_alpha: style.background_alpha). fill_color(style.background_color).rectangle(x, y, width, height).fill end end style.underlays.draw(canvas, x, y, self) if style.underlays? style.border.draw(canvas, x, y, width, height) if style.border? draw_content(canvas, x + reserved_width_left, y + reserved_height_bottom) style..draw(canvas, x, y, self) if style. end |
#empty? ⇒ Boolean
Returns true if no drawing operations are performed.
220 221 222 223 224 225 226 |
# File 'lib/hexapdf/layout/box.rb', line 220 def empty? !(@draw_block || (style.background_color? && style.background_color) || (style.underlays? && !style.underlays.none?) || (style.border? && !style.border.none?) || (style. && !style..none?)) end |
#fit(available_width, available_height, _frame) ⇒ Object
Fits the box into the Frame and returns true if fitting was successful.
The default implementation uses the whole available space for width and height if they were initially set to 0. Otherwise the specified dimensions are used.
160 161 162 163 164 |
# File 'lib/hexapdf/layout/box.rb', line 160 def fit(available_width, available_height, _frame) @width = (@initial_width > 0 ? @initial_width : available_width) @height = (@initial_height > 0 ? @initial_height : available_height) @fit_successful = (@width <= available_width && @height <= available_height) end |
#split(available_width, available_height, frame) ⇒ Object
Tries to split the box into two, the first of which needs to fit into the available space, and returns the parts as array.
If the first item in the result array is not nil, it needs to be this box and it means that even when #fit fails, a part of the box may still fit. Note that #fit should not be called before #draw on the first box since it is already fitted. If not even a part of this box fits into the available space, nil should be returned as the first array element.
Possible return values:
- [self]
-
The box fully fits into the available space.
- [nil, self]
-
The box can't be split or no part of the box fits into the available space.
- [self, new_box]
-
A part of the box fits and a new box is returned for the rest.
This default implementation provides the basic functionality based on the #fit result that should be sufficient for most subclasses; only #split_content needs to be implemented if necessary.
183 184 185 186 187 188 189 190 191 192 |
# File 'lib/hexapdf/layout/box.rb', line 183 def split(available_width, available_height, frame) if @fit_successful [self, nil] elsif (style.position != :flow && (@width > available_width || @height > available_height)) || content_height == 0 || content_width == 0 [nil, self] else split_content(available_width, available_height, frame) end end |
#split_box? ⇒ Boolean
Returns true if this is a split box, i.e. the rest of another box after it was split.
135 136 137 |
# File 'lib/hexapdf/layout/box.rb', line 135 def split_box? @split_box end |
#supports_position_flow? ⇒ Boolean
Returns false since a basic box doesn't support the 'position' style property value :flow.
140 141 142 |
# File 'lib/hexapdf/layout/box.rb', line 140 def supports_position_flow? false end |