Class: HexaPDF::Type::GraphicsStateParameter
- Inherits:
-
Dictionary
- Object
- Object
- Dictionary
- HexaPDF::Type::GraphicsStateParameter
- Defined in:
- lib/hexapdf/type/graphics_state_parameter.rb
Overview
Represents a graphics state parameter dictionary.
This dictionary can be used to define most graphics state parameters that are available. Some parameters can only be set by an operator, some only by the dictionary but most by both.
See: PDF2.0 s8.4.5, s8.1
Constant Summary
Constants included from DictionaryFields
DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate
Method Summary
Methods inherited from Dictionary
#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type