Class: HexaPDF::Type::ViewerPreferences
- Inherits:
-
Dictionary
- Object
- Object
- Dictionary
- HexaPDF::Type::ViewerPreferences
- Defined in:
- lib/hexapdf/type/viewer_preferences.rb
Overview
Represents the PDF's viewer preferences dictionary which defines how a document should be presented on screen or in print.
This dictionary is linked via the /ViewerPreferences entry from the Type::Catalog.
See: PDF2.0 s12.2, Catalog
Constant Summary
Constants included from DictionaryFields
DictionaryFields::Boolean, DictionaryFields::PDFByteString, DictionaryFields::PDFDate
Method Summary
Methods inherited from Dictionary
#[], #[]=, define_field, define_type, #delete, #each, each_field, #empty?, field, #key?, #to_hash, type, #type